@@ -16,7 +16,7 @@ use alloc::vec::Vec;
 
 use zune_core::bytestream::ZByteReaderTrait;
 use zune_core::colorspace::ColorSpace;
-use zune_core::log::{debug, error, trace, warn};
+use zune_core::log::{debug, trace, warn};
 
 use crate::components::Components;
 use crate::decoder::{GainMapInfo, ICCChunk, JpegDecoder, MAX_COMPONENTS};
@@ -314,6 +314,8 @@ pub(crate) fn parse_sos<T: ZByteReaderTrait>(
     }
 
     // consume spec parameters
+    image.scan_subsampled = false;
+
     for i in 0..ns {
         let id = image.stream.read_u8_err()?;
 
@@ -347,9 +349,21 @@ pub(crate) fn parse_sos<T: ZByteReaderTrait>(
             )));
         }
 
-        image.components[usize::from(j)].dc_huff_table = usize::from((y >> 4) & 0xF);
-        image.components[usize::from(j)].ac_huff_table = usize::from(y & 0xF);
+        let component = &mut image.components[usize::from(j)];
+        component.dc_huff_table = usize::from((y >> 4) & 0xF);
+        component.ac_huff_table = usize::from(y & 0xF);
         image.z_order[i as usize] = j as usize;
+
+        if component.vertical_sample != 1 || component.horizontal_sample != 1 {
+            image.scan_subsampled = true;
+        }
+
+        trace!(
+            "Assigned huffman tables {}/{} to component {j}, id={}",
+            image.components[usize::from(j)].dc_huff_table,
+            image.components[usize::from(j)].ac_huff_table,
+            image.components[usize::from(j)].id,
+        );
     }
 
     // Collect the component spec parameters
@@ -415,7 +429,7 @@ pub(crate) fn parse_sos<T: ZByteReaderTrait>(
 pub(crate) fn parse_app13<T: ZByteReaderTrait>(
     decoder: &mut JpegDecoder<T>
 ) -> Result<(), DecodeErrors> {
-    const IPTC_PREFIX: &[u8] = b"Photoshop 3.0";
+    const IPTC_PREFIX: &[u8] = b"Photoshop 3.0\0";
     // skip length.
     let mut length = usize::from(decoder.stream.get_u16_be());
 
@@ -449,12 +463,13 @@ pub(crate) fn parse_app14<T: ZByteReaderTrait>(
     if length < 2 {
         return Err(DecodeErrors::FormatStatic("Too small APP14 length"));
     }
-    if length < 14 {
-        return Err(DecodeErrors::FormatStatic(
-            "Too short of a length for App14 segment"
-        ));
-    }
+
     if decoder.stream.peek_at(0, 5)? == b"Adobe" {
+        if length < 14 {
+            return Err(DecodeErrors::FormatStatic(
+                "Too short of a length for App14 segment"
+            ));
+        }
         // move stream 6 bytes to remove adobe id
         decoder.stream.skip(6)?;
         // skip version, flags0 and flags1
@@ -477,11 +492,9 @@ pub(crate) fn parse_app14<T: ZByteReaderTrait>(
         // version =  5
         // transform = 1
         length = length.saturating_sub(14);
-    } else if decoder.options.strict_mode() {
-        return Err(DecodeErrors::FormatStatic("Corrupt Adobe App14 segment"));
     } else {
+        warn!("Not a valid Adobe APP14 Segment, skipping {} bytes", length);
         length = length.saturating_sub(2);
-        error!("Not a valid Adobe APP14 Segment");
     }
     // skip any proceeding lengths.
     // we do not need them
